/***********************************************************************
 
 Spiritus Runner - 2D Runner
 Gestartet am: 22.03.2023
 Mitglieder: Nicolò Tonarelli, Mika Pfeifer, Andrea Krauss
 
 Hauptentwickler Processing: Nicolò
 Hauptentwickler Website: Andrea Krauss & Mika Pfeifer
 
 *************************************************************************/



//************************************************************************************************************************************
/*************************************************************************************************************************************
 **                                                                                                                                  **
 **                                                        MAIN                                                                      **
 **                                                                                                                                  **
 **   Hier befinden sich alle Funktionen / Variablen, die für das grobe Spielsystem verantwortlich sind                              **
 **                                                                                                                                  **
 **************************************************************************************************************************************/
//************************************************************************************************************************************
import processing.sound.*;

// Schriftarten
PFont mono;

// Verschiedene Spielzustände
int gameState = 0; // 0 = Menü, 1 = Spiel, 2 = settings, 3 = Shop, 4 = GameOver, 5 = Animation

// Menü Variablen / Settings
int buttonWidth  = 250;
int buttonHeight = 100;
int textSize = 50;
int textHeight = textSize/2;
color menuThemeColor = #F5DC19;

// Zeit / Tickers etc.
int ticker = 0;
int tickerMemory;

// Bilder
PImage background;
PImage coinPreview;
PImage left;
PImage up;

// Sounds
SoundFile backgroundMusic;
SoundFile animationMusic;

SoundFile hoverSound;
SoundFile gameOverSound;
SoundFile cashSound;
SoundFile victorySound;

SoundFile jumpSound;
SoundFile damageSound;
SoundFile levelChangeSound;
SoundFile coinSound;

// Array wo die Musik Dateien beinhaltet
SoundFile[] music = new SoundFile[4];

// Buttons
Button playButton;
Button shopButton;
Button quitButton;
Button settingsButton;
Button settingsDelete;
Button backButton;

// Shop Buttons
Button shopItem1;
Button shopItem2;
Button shopItem3;

// Shop Preise
int shopItem1Price = 100;
int shopItem2Price = 300;
int shopItem3Price = 50;

// Wert für ob der Game Over ein Sieg war
boolean isVictory = false;

// Liste an Bildern von Spielerlaufanimation
ArrayList<PImage> playerImages = new ArrayList<PImage>();

/*/************************************************************************************************************************************
 **                                                                                                                                  **
 **                                                        SETUP (Startvariablen werden gesetzt)                                     **
 **                                                                                                                                  **
//************************************************************************************************************************************/
void setup() {
  fullScreen(); //Vollbild


  // Bilder werden geladen
  background = loadImage("images/background.jpeg");
  coinPreview = loadImage("images/coin.png");
  left = loadImage("images/debug/left.png");
  up = loadImage("images/debug/up.png");

  // Fonts
  mono = createFont("fonts/pixel.ttf", 128);

  // Sounds werden geladen
  backgroundMusic = new SoundFile(this, "sounds/background-music.wav");
  hoverSound = new SoundFile(this, "sounds/hover.wav");
  animationMusic = new SoundFile(this, "sounds/animation-music.wav");
  gameOverSound = new SoundFile(this, "sounds/gameover.wav");
  cashSound = new SoundFile(this, "sounds/cash.wav");
  victorySound = new SoundFile(this, "sounds/victory.wav");
  
  jumpSound = new SoundFile(this, "sounds/jump.wav");
  damageSound = new SoundFile(this, "sounds/damage.wav");
  levelChangeSound = new SoundFile(this, "sounds/levelchange.wav");
  coinSound = new SoundFile(this, "sounds/coin.wav");
  
  // Die 4 Muskdateien werden geladen
  for(int i = 1; i <= 4; i++) {
    music[i-1] = new SoundFile(this, "sounds/music"+i+".wav");
  }
  

  // Programminhalte werden generiert
  createScreen(true);                                                                                              

  // Settingsdelete Button wird geladen
  settingsDelete = new Button("Spielstand löschen", width/2, height/2, menuThemeColor, #D3C02D, #FFF079, buttonWidth*3, buttonHeight, 0, textSize); // Settingsbutton

  // Spielerbilder werden in die ArrayList geladen
  for (int i = 1; i <= 5; i++) {                                           // For Schleife von 1 - 5
    PImage tempImage = loadImage("images/player"+i+".png");                // Temporäres Bild
    tempImage.resize(int(tempImage.width*2/3), int(tempImage.height*2/3)); // Bild wird resized
    playerImages.add(tempImage);                                           // Bild wird zur Liste hinzugefügt
  }
}

/*/************************************************************************************************************************************
 **                                                                                                                                  **
 **                                                        DRAW (Spielstoppuhr um 1 höher und createScreen())                        **
 **                                                                                                                                  **
//************************************************************************************************************************************/
void draw() {
  ticker++;            // Zeitrechnung für Animationen & Systeme
  createScreen(false); // Programminhalte werden generiert
}


/*/************************************************************************************************************************************
 **                                                                                                                                  **
 **                                                        createScreen (Funktion, die alle States / Spielzustände verwaltet         **
 **                              Je nach boolean gameStateSwitch werden Startwerte gesetzt oder nur Funktionen ausgeführt            **
 **                                                                                                                                  **
//************************************************************************************************************************************/
void createScreen(boolean gameStateSwitch) {
  switch(gameState) {

  case 0: ///////////////////////////////////////////////////////////////////////// GAMESTATE: MENU
      if (!gameStateSwitch)  return;               // -> Funktion nur 1x ausgeführt

    // Menü Background
    background.resize(1920, 1080);                 // Resize
    image(background, 0, 0);                       // Menühintergrund Bild

    restartMusic();                                // Nur Menümusik wird gestartet
    createButtons();                               // Buttons werden gezeichnet
    drawMainMenu();                                // Menütitel werden gezeichnet
    loadGameData();                                // Daten aus data.txt werden geladen
    loadScores();                                  // Münzen & Highscore werden angezeigt
    break;

  case 1: ///////////////////////////////////////////////////////////////////// GAMESTATE: GAME
    if (gameStateSwitch) {                         // (Nur 1x mal)
      background(#FFFFFF);                         // Hintergrundfarbe weiss
      ticker = 0;                                  // Stoppuhr auf 0
      tickerMemory = ticker;                       // Stopppuhrspeicher auf ticker
      setupGame();                                 // Spielstartwerte
      animationMusic.stop();                       // Erklärungsmusik wird gestoppt
    }
    gameHandler();                                 // Spielfunktionen
    break;
  case 2: ///////////////////////////////////////////////////////////////////////////// SETTINGS
    if (!gameStateSwitch)  return;                // nur 1x mal

    // Menü Background
    background.resize(1920, 1080);
    image(background, 0, 0);

    // Settingsbutton wird angezeigt
    settingsDelete.draw(1, true);                // settingsDelete-Button wird im Knopfzustand 1 (unberührt) gezeichnet

    // Menü Musik wird geändert
    backgroundMusic.stop();                     
    animationMusic.play();
    animationMusic.loop();

    // Zurück/Back Button wird angezeigt
    backButton.draw(1, true);                    // Zurückbutton im Knopfzustand 1 gezeichnet
    break;

  case 3: /////////////////////////////////////////////////////////////////////////////// SHOP

    if (gameStateSwitch) {                      // nur 1x mal
      background(#FFFFFF);  
      backgroundMusic.stop();
      animationMusic.play();
      animationMusic.loop();

      backButton.draw(1, true);                // Backbuttobn im Zustand 1 gezeichnet        
    }

    break;
  case 4: /////////////////////////////////////////////////////////////////////////// GAME OVER

    if (gameStateSwitch) {                    // nur 1xmal
      saveGameData();
      ticker = 0;                             // Stoppuhr auf 0
      tickerMemory = ticker;                  // Stoppuhrspeicher auf Stoppuhr (0)

      if (isVictory) {                        // Falls man gewonnen hat
        victorySound.play();                  //   Siegmusik wird abgespielt
      } else gameOverSound.play();            //   GameOvermusik wird abgespielt
    }
    currentMusic.stop();                      // Spielmusik wird gestoppt
    drawGameOver();                           // GameOver wird gezeichnet

    if (ticker >= 100) {                      // Nach 100 Frames 
      if (!gameOverSound.isPlaying()) {       // GameOverSound gestoppt
        backButton.draw(1, true);             // Backbutton wird angezeigt
      }
    }

    break;
  case 5: /////////////////////////////////////////////////////////////// ANIMATIONS LOADING SCREEN

    // VOREINSTELLUNGEN UND TIMER STARTEN
    if (gameStateSwitch) {                    // nur 1x
      frameRate(200);                         // Frame Rate auf 200
      tickerMemory = ticker;

      backgroundMusic.stop();
      animationMusic.play();
      animationMusic.loop();
      //backgroundMusic.stop();
    }


    // KÄSTCHEN ANIMATION
    if (ticker - 570  < tickerMemory) {      // Nach 570 Frames Loading Screen
      fill(menuThemeColor);
      noStroke();
      square(int(random(0, width)), int(random(0, height)), int(random(0, 400)));

      fill(#FFFFFF);
      textSize(textSize*2);
      text("Loading...", width/2, height/2);
    }

    // Erste mal Hintergrund färben (gelb)
    if (ticker - 571 == tickerMemory) {     // Nach 571 Frames Fenster färben
      background(menuThemeColor);
    }
    if (ticker - 600 == tickerMemory) {      // Nach 600 Frames Steuerung angzeigen
      background(menuThemeColor);

      stroke(#FFFFFF);                      // Linie
      strokeWeight(5);
      line(620, 230, 1290, 230);

      stroke(#FFFFFF);                       // Linie
      strokeWeight(5);
      line(0, height-100, width, height-100);

      stroke(#FFFFFF);                        // Linie
      strokeWeight(5);
      line(0, 100, width, 100);

      fill(#FFFFFF);                          // Steuerungstext
      text("Steuerung", width/2, height/5*1);
      textSize(textSize);                  // Springen Text
      text("[Leertaste]: \u2191 - Springen", width/2, height/6*2);
      text("[Pfeiltaste]: \u2193 - Ducken", width/2, height/6*3);
      text("[Ziel]: \u2193 - Laufe durch alle Levels!", width/2, height/6*4);
      text("[!]: \u2193 - Meide Objekte & Lehrer!", width/2, height/6*5);

    }

    if (ticker - 1400 == tickerMemory) {     // Nach 1400 Frames Lets Go anzeigen
      background(menuThemeColor);

      fill(#FFFFFF);
      text("Lets go!", width/2, height/2);
    }

    if (ticker - 1750 == tickerMemory) {    // Nach 1750 Frames Bildschirm färben
      background(menuThemeColor);
    }

    if (ticker - 1900 == tickerMemory) {    // Nach 1900 Frames State wechseln
      gameState = 1;
      frameRate(60);
      animationMusic.stop();
      createScreen(true);
    }
    break;
  }///////////////////////////////////////////////////////////////////////////////////////
}

/*/************************************************************************************************************************************
 **                                                                                                                                  **
 **                                                        Funktion, die zu Beginn des Menü geladen wird                             **
 **                                                                                                                                  **
//************************************************************************************************************************************/
void restartMusic() {                     
  backgroundMusic.play();
  backgroundMusic.loop();
  backgroundMusic.amp(0.1);

  animationMusic.stop();
  gameOverSound.stop();
}

/*/************************************************************************************************************************************
 **                                                                                                                                  **
 **                                                        Startbuttons (Buttons werden geladen)                                     **
 **                                                                                                                                  **
//************************************************************************************************************************************/
void createButtons() {
  playButton = new Button("PLAY", width/2, height/4*1, menuThemeColor, #D3C02D, #FFF079, buttonWidth, buttonHeight, 0, textSize);
  playButton.draw(1, true);

  shopButton = new Button("SHOP", width/2, height/4*2, menuThemeColor, #D3C02D, #FFF079, buttonWidth, buttonHeight, 0, textSize);
  shopButton.draw(1, true);

  quitButton = new Button("QUIT", width/2, height/4*3, menuThemeColor, #D3C02D, #FFF079, buttonWidth, buttonHeight, 0, textSize);
  quitButton.draw(1, true);

  shopItem1 = new Button("Doppelsprung (100$)", width/2, height/4*1, menuThemeColor, #D3C02D, #FFF079, buttonWidth*3, buttonHeight, 0, textSize);
  //shopItem1.draw(1, true);

  shopItem2 = new Button("x2 Leben (300$)", width/2, height/4*2, menuThemeColor, #D3C02D, #FFF079, buttonWidth*3, buttonHeight, 0, textSize);
  //shopItem2.draw(1, true);

  shopItem3 = new Button("Mütze kaufen (50$)", width/2, height/4*3, menuThemeColor, #D3C02D, #FFF079, buttonWidth*3, buttonHeight, 0, textSize);
  //shopItem3.draw(1, true);


  settingsButton = new Button("SETTINGS", 130, height-120, menuThemeColor, #D3C02D, #FFFFFF, 200, 50, 0, 30);
  settingsButton.draw(1, true);

  backButton = new Button("BACK", 1840, 1030, menuThemeColor, #D3C02D, #FFFFFF, 200, 50, 0, 30);
}

/*/************************************************************************************************************************************
 **                                                                                                                                  **
 **                                                        Score & Highscore werden angezeigt                                        **
 **                                                                                                                                  **
//************************************************************************************************************************************/
void loadScores() {  
  fill(menuThemeColor);
  textSize(textSize/2);
  textAlign(RIGHT);
  text(highscore + " Highscore", 1700, 40);

  fill(menuThemeColor);
  textSize(textSize/2);
  textAlign(RIGHT);
  text(coins + " $", 1700, 40+(textSize/1.5));

  coinPreview.resize(70, 70);  
  image(coinPreview, 1710, 15);
}

/*/************************************************************************************************************************************
 **                                                                                                                                  **
 **                                                        Hauptmenü Titel wird gezeichnet                                           **
 **                                                                                                                                  **
//************************************************************************************************************************************/
void drawMainMenu() {
  stroke(menuThemeColor);
  strokeWeight(10);
  line(100, 555, 660, 555);

  stroke(menuThemeColor);
  strokeWeight(10);
  line(1295, 555, 1770, 555);

  fill(menuThemeColor);
  textSize(textSize*2);
  text("SPIRITUS", width/10*2, height/2);

  fill(menuThemeColor);
  textSize(textSize*2);
  text("RUNNER", width/10*8, height/2);
}

/*/************************************************************************************************************************************
 **                                                                                                                                  **
 **                                                        Game Over wird gezeichnet                                                 **
 **                                                                                                                                  **
//************************************************************************************************************************************/
void drawGameOver() {
  if(isVictory) { background(menuThemeColor); } else { background(255,0,0); } // Je Nach GameOverTyp wird die Hintergrundfarbeanders
  textAlign(CENTER);
  fill(#FFFFFF);
  textSize(200);
  text("Game Over", width/2, height/2);

  textAlign(CENTER);
  fill(#FFFFFF);
  textSize(50);
  text("Score: " + score + " Highscore: " + highscore, width/2, 650);
}
