//***************************************************************************************************************************
/****************************************************************************************************************************
 **                                                                                                                         **
 **                       GAME                                                                                              **
 **                                                                                                                         **
 **   Hier befinden sich alle Funktionen die während des Spiels ausgeführt werden                                           **
 **                                                                                                                         **
 ****************************************************************************************************************************/
//***************************************************************************************************************************

// Münzenbild
PImage coinImage;

int score; // Punktzahl

boolean canJump; // Sprungstatus des Spielers (ob der wieder springen kann)
boolean showHitboxes; // Debugmode (Zeigt die Kollisionsboxen an, kann mit 'd' aktiviert/deaktiviert werden)

PImage bg1, bg2, bg3, bg3R; // Hintergründe
PImage accessoirHelmet;     // Mützenbild

float bg1Speed = 0.2;       // Hintergrund 1 Geschwindigkeit
float bg2Speed = 0.5;       // Hintergrund 2 Geschwindigkeit
float bg3Speed = 1.0;       // Hintergrund 3 Geschwindigkeit

float bg1Offset = 0;       // Hintergrund 1 offset
float bg2Offset = 0;       // Hintergrund 2 offset
float bg3Offset = 0;       // Hintergrund 3 offset

int gameTicker;            // Stoppuhr (für Timer, Funktionen, Cooldowns...)
int tickSaveTheme;         // Stoppuhrspeicher (für Timer, Funktionen, Cooldowns...)
int herzen;                // Anzahl an Herzen
int levelTime = 20;        // Zeit, die ein Level benötigt
int theme;                 // Aktuelle ID vom Level (Mensa, Zimmer, Gym...)
int jumpCount = 0;         // Anzahl bereits gemachte Sprünge (um Doppelsprung zu berechnen)
 
int currentMusicNumber;    // Aktuelle Musik ID
SoundFile currentMusic;    // Aktuelle Musik Datei

ArrayList<Obstacle> obstacles = new ArrayList<Obstacle>(); // Liste von allen Gegern, Münzen & Objekten
ArrayList<Integer> alreadyThemesBeen = new ArrayList<Integer>(); // Liste von den Levels die man schon durchquert hat

Player player;              // Spielerobjekt

/*/************************************************************************************************************************************
 **                                                                                                                                  **
 **                                                        Funktion, die die ganzen Werte, Bilder etc. zu Spielbeginn ladet          **
 **                                                                                                                                  **
 //************************************************************************************************************************************/
void setupGame() {

  // Hintergrundbilder werden geladen und resized
  bg1 = loadImage("images/themes/tree_layer.png");
  bg1.resize(width, height);
  bg2 = loadImage("images/themes/mountain_layer.png");
  bg2.resize(width, height);
  bg3 = loadImage("images/themes/wallpaper.png");
  bg3.resize(width, height);
  bg3R = loadImage("images/themes/wallpaper.png");
  bg3.resize(width, height);

  // Geschwindigkeit Hintergründe
  bg1Speed = 5;
  bg2Speed = 3;
  bg3Speed = 1;

  // Timer / Stoppuhren zurückgesetzt
  gameTicker = 0;
  tickSaveTheme = 0;
  jumpCount = 0;

  // Annzahl Herzen
  herzen = 10;
  if (boughtItem2) herzen*=2; // Falls x2 Herzen gekauft wurde (im Shop)

  // Zufälliges Level wird ausgewählt
  theme = Math.round(random(1, 5));
  alreadyThemesBeen.clear();  // Liste zurückgesetzt
  alreadyThemesBeen.add(theme); // Neues Level zur Liste

  score = 0;                    // Score zurückgesetzt
  showHitboxes = false;         // Debugmode deaktiviert
  isVictory = false;            // Sieg wird deaktiviert

  // Zufällige Musik wird ausgewählt und abgespielt
  currentMusicNumber = int(random(1, 4));
  currentMusic = music[currentMusicNumber-1];

  // Spielerobjekt wird definiert
  player = new Player(width/2, height/2, 0.0, 0.6, 1, 4, true);//(float x, float y, float speed, float gravity, int currentImage, int maxImages, boolean canJump)

  // Falls Item 3 im  Shop gekauft wurde, wird der Helm geladen
  if (boughtItem3) {
    accessoirHelmet = loadImage("./images/extra/helmets/" + Math.round(random(1, 6)) + ".png");
  }
}


/*/************************************************************************************************************************************
 **                                                                                                                                  **
 **                                                        Funktion, die jeden Frame ausgeführt wird und alle Spielabläufe verwaltet **
 **                                                                                                                                  **
 //************************************************************************************************************************************/
void gameHandler() {
  gameTicker++; // Stoppuhr um 1 höher

  background(#FFFFFF);   // Hintergrund zeichnen
  player.move();         // Spieler bewegen
  moveBackgrounds();     // Verschiebung des hintersten Hintergrundbild von rechts nach links


  if (player.y >= height - player.playerImg.height/2) {  // Spieler Spring Animation, falls er am Springen ist / über dem Boden ist
    player.displayJump();
  }

  player.display();  // Spieler wird gezeichnet
  displayPoints();   // Punktzahl anzeigen
  obstacleHandler(); // Hindernis bewegen und zeichnen
  levelChanger();    // Nach einiger Zeit wird das Level gewechselt / Victory (Game over) aktiviert
  spawnObstacles();  // Gegner werden erschaffen
  spawnCoin();       // Münzen werden erschaffen
  drawHearts();      // Herzensanzeige wird gezeichnet
  checkIfMusicEnded(); // Musik wird neugstartet falls gestoppt
}

void checkIfMusicEnded() {
  if (!(currentMusic.isPlaying())) {
    int newMusicNumber = int(random(1, 4));
    do {
      newMusicNumber=int(random(1, 4));
    } while (newMusicNumber==currentMusicNumber);

    currentMusicNumber = newMusicNumber;
    currentMusic = music[currentMusicNumber-1];
    currentMusic.play();
  }
}

/*/************************************************************************************************************************************
 **                                                                                                                                  **
 **                                                        Funktion, die das Level ändert oder Game Over (bei Sieg) aktiviert        **
 **                                                                                                                                  **
 //************************************************************************************************************************************/
void levelChanger() {



  if (tickSaveTheme / 60f + levelTime < ticker / 60f ) { // Alle 'levelTime' Sekunden (60 Frames) wird das Level gewechselt

    if (alreadyThemesBeen.size() == 5) { // Falls alle Level durchgespielt wurden
      isVictory=true;
      currentMusic.stop();
      doGameOver();  // Statewechsel / Game Over aktiviert
      return;
    }

    int n;

    do { // Solange kein neues Level gefunden wurde, wird der Random Integer neugeneriert
      n =  Math.round(random(1, 5));
    } while (alreadyThemesBeen.contains(n));

    theme = n;
    alreadyThemesBeen.add(theme);
    tickSaveTheme = ticker;
  }
}

/*/************************************************************************************************************************************
 **                                                                                                                                  **
 **                                                        Funktion, die die Hintergründe verschiebt                                 **
 **                                                                                                                                  **
 //************************************************************************************************************************************/
void moveBackgrounds() {
  bg3Offset -= bg3Speed;
  bg3Offset = bg3Offset % bg3.width;
  for (float x = bg3Offset; x > -width; x -= bg3.width) { // Hintergrund wird bewegt sobald er das Fenster verlässt
    image(bg3, x, 0);
    image(bg3R, width+bg3Offset, 0);
    if (showHitboxes) showDebugRect(color(255, 0, 0), int(x), 0, bg3.width, bg3.height);
    if (showHitboxes) showDebugRect(color(255, 0, 0), int(width+bg3Offset), 0, bg3R.width, bg3R.height);
  }

  // Verschiebung des zweiten Hintergrundbild von rechts nach links
  bg2Offset -= bg2Speed;
  bg2Offset = bg2Offset % bg2.width;
  for (float x = bg2Offset; x > -width; x -= bg2.width) { // Hintergrund wird bewegt sobald er das Fenster verlässt
    image(bg2, x, 0);
    image(bg2, width+bg2Offset, 0);
    if (showHitboxes) showDebugRect(color(0, 255, 0), int(x), 0, bg2.width, bg2.height);
    if (showHitboxes) showDebugRect(color(0, 255, 0), int(width+bg2Offset), 0, bg2.width, bg2.height);
  }

  // Verschiebung des dritten Hintergrundbild von rechts nach links
  bg1Offset -= bg1Speed;
  bg1Offset = bg1Offset % bg1.width;
  for (float x = bg1Offset; x > -width; x -= bg1.width) { // Hintergrund wird bewegt sobald er das Fenster verlässt
    image(bg1, x, 0);
    image(bg1, width+bg1Offset, 0);
    if (showHitboxes) showDebugRect(color(0, 255, 0), int(x), 0, bg1.width, bg2.height);
    if (showHitboxes) showDebugRect(color(0, 0, 255), int(width+bg1Offset), 0, bg2.width, bg2.height);
  }
}

/*/************************************************************************************************************************************
 **                                                                                                                                  **
 **                                                        Score wird angezeigt                                                      **
 **                                                                                                                                  **
 //************************************************************************************************************************************/
void displayPoints() {
  textAlign(RIGHT);
  textSize(32);
  fill(255);
  text("Score: " + score, width - 20, 40);
}

/*/************************************************************************************************************************************
 **                                                                                                                                  **
 **                                                        Anzahl Herzen wird mit dem Symbol \u2764 gezeichnet                       **
 **                                                                                                                                  **
 //************************************************************************************************************************************/
void drawHearts() {
  String finalHeartText = "";
  for (int i = 0; i < herzen; i++) finalHeartText += "\u2764"; // Von 0 - i (For Schleife) wird dem endgültigen String 'finalHeartText' das Herzsymbol hinzugefügt

  fill(255, 0, 0);
  textSize(80);
  textAlign(LEFT);
  text(finalHeartText, 40, 80); // Finale String wird angezeigt
}

/*/************************************************************************************************************************************
 **                                                                                                                                  **
 **                                                        Funktion die deen Game Over Screen einleitet                              **
 **                                                                                                                                  **
 //************************************************************************************************************************************/
void doGameOver() {
  gameState = 4; // State wird gewechselt
  createScreen(true);
  currentMusic.stop(); // Spielmusik wird gestoppt
}


/*/************************************************************************************************************************************
 **                                                                                                                                  **
 **                                                Funktion fürs Debuggen, die Ränder von Rechtecken zeichnet                        **
 **                                                                                                                                  **
 //************************************************************************************************************************************/
void showDebugRect(color c, int x, int y, int w, int h) {
  stroke(c);
  strokeWeight(3);
  noFill();
  rectMode(CORNER);
  rect(x, y, w, h);
}

/*/************************************************************************************************************************************
 **                                                                                                                                  **
 **                                             Spielerobjekt das den Sprung, Bewegung & Animation vom Spieler verwaltet             **
 **                                                                                                                                  **
 //************************************************************************************************************************************/
class Player {

  float x, y, speed, gravity; // X, Y, Sprungeschwindigkeit, Schwerkraft
  int currentImage;           // Aktuelles Bild der Animation
  int maxImages;              // Wie viele versch. Spielerversionen es gibt (Animation)
  PImage playerImg;           // Aktuelles Spielerbild was angezeigt wird
  boolean canJump;            // Boolean für ob der Spieler gerade springen kann
  boolean crouch;             // Ob der Spieler gerade am Crouchen ist

  // Konstruktor
  Player(float x, float y, float speed, float gravity, int currentImage, int maxImages, boolean canJump) {
    this.x = x;
    this.y = y;
    this.speed = speed;
    this.gravity = gravity;
    this.currentImage = currentImage;
    this.maxImages = maxImages;
    this.canJump = canJump;
    crouch = false;

    this.playerImg = playerImages.get(0);
  }

  void display() {
    if (ticker % 6 == 0) currentImage = currentImage+1>=maxImages ? 1 : currentImage+1; // Alle 6 Frames (wenn der frameCount durch 6 teilbar ist) wird das Spielerbild der Animation mit den nächsten getauscht
    this.playerImg = playerImages.get(currentImage);                                    // Neues Spielerbild wird gesetzt
    if (crouch) this.playerImg = playerImages.get(4);                                   // Falls der Spieler am Schleichen ist, wird der Schleichbild angezeigt

    // Falls der Spieler den Hut gekauft hat im Shop wird er hier angezeigt
    imageMode(CENTER);
    if (boughtItem3) image(accessoirHelmet, this.x, this.y-25);
    image(this.playerImg, this.x, this.y);
    imageMode(CORNER);
  }

  // Falls man Leertaste drückt
  void jump() {
    this.speed = -20;                                         // Spielersprung (wirkt sich auf Y-Koordinate des Spielers aus)  wird auf -20 gesetzt -> Spieler geht hoch
    if (!boughtItem1 || jumpCount >= 1) this.canJump = false; // Spieler kann nicht erneut springen, bis er den Boden erreicht hat
    jumpSound.play();                                         // Sprungsound
  }

  void move() {
    this.y += this.speed;                                     // Spielersprung wirkt sich auf Y aus
    this.speed += this.gravity;                               // Schwerkraft wirkt sich auf Spielersprung aus -> Spieler geht wieder runter
    this.speed = constrain(this.speed, -10, 10);              // Sprungkraft wird begrenzt
    if (this.x < width/2) this.x++;                           // Falls der Spieler nicht in der Mitte ist, wird er sich dahin wieder bewegen (NICHT IN VERWENDUNG)
    if (this.x < 0) {                                         // Falls der Spieler aus dem Fenster geht kriegt er Schaden und wird wieder verschoben
      herzen--;
      this.x = width/2;
      this.y = 500;
    }
  }


  void displayJump() {                                        // Sprung wird angezeigt
    this.y = height - this.playerImg.height/2;                // Y-Koordinate wird nochmals korrigiert
    this.speed = 0;                                           // Sprungkraft auf 0
    this.canJump = true;                                      // Spieler kann erneut springen, da er den Boden erreicht hat
    jumpCount=0;                                              // jumpCount auf 0
  }
}
